/*
 *   Copyright 2012 Brian Rosenberger (Brutex Network)
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
package net.brutex.xservices.types;

import java.util.TimeZone;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlType;

import net.brutex.xservices.util.BrutexNamespaces;

/**
 * Different pre-defined date formats.
 *
 * @author Brian Rosenberger, bru@brutex.de
 */
@XmlType(name=TimeZoneType.XML_NAME, namespace=BrutexNamespaces.WS_XSERVICES)
public class TimeZoneType {
	public static final String XML_NAME = "timezone";
	private String id;
	private String displayname;
	private TimeZone timezone;
	private long offset;
	
	public TimeZoneType(TimeZone timezone) {
		this.timezone = timezone;
		this.id = timezone.getID();
		this.displayname = timezone.getDisplayName();
		this.offset = timezone.getRawOffset()/1000;		
	}
	public TimeZoneType() {
		this(TimeZone.getDefault());
	}

	/**
	 * @return the id
	 */
	@XmlElement
	public String getId() {
		return id;
	}

	/**
	 * @return the displayname
	 */
	@XmlElement
	public String getDisplayname() {
		return displayname;
	}

	/**
	 * @return the timezone
	 */
	public TimeZone getTimezone() {
		return timezone;
	}

	/**
	 * @return the offset
	 */
	@XmlElement
	public long getOffset() {
		return offset;
	}
	
	
	
}
