/*
 *   Copyright 2013 Brian Rosenberger (Brutex Network)
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package net.brutex.xservices.ws;

import java.util.ArrayList;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.xml.bind.annotation.XmlElement;

import net.brutex.xservices.types.StringMatchType;
import net.brutex.xservices.types.StringReplaceType;
import net.brutex.xservices.types.StringSplitType;

import org.apache.cxf.annotations.WSDLDocumentation;

/**
 * @author Brian Rosenberger, bru(at)brutex.de
 *
 */
@WebService(targetNamespace="http://ws.xservices.brutex.net")
public abstract interface StringService
{
  public static final String SERVICE_NAME = "StringService";
  public static final String OPERATION_REPLACEREGEX = "replaceRegEx";
  public static final String OPERATION_MATCHREGEX = "matchRegEx";
  public static final String OPERATION_ENCODETOENTITIES = "encodeToXMLEntities";
  public static final String PARAM_STRING = "string";
  public static final String PARAM_SEARCH = "search";
  public static final String PARAM_REPLACE = "replace";
  public static final String PARAM_FLAGS = "regexflags";

  @WebMethod(operationName="replaceRegEx")
  @WSDLDocumentation("Store text based data")
  public abstract StringReplaceType replaceRegEx(
		  @WebParam(name="string") String paramString1, 
		  @WebParam(name="search") String paramString2, 
		  @WebParam(name="replace") String paramString3, 
		  @WebParam(name="regexflags") String paramString4)
    throws XServicesFault;

  @WebMethod(operationName="matchRegEx")
  @WSDLDocumentation("Match text based data")
  public abstract StringMatchType matchRegEx(
		  @WebParam(name="string") String paramString1, 
		  @WebParam(name="search") String paramString2, 
		  @WebParam(name="regexflags") String paramString3)
    throws XServicesFault;

  @WebMethod(operationName="encodeToXMLEntities")
  @WSDLDocumentation("Match text based data")
  public abstract String encodeToXMLEntities(
		  @WebParam(name="string") @XmlElement(required=true) String paramString)
    throws XServicesFault;
  
  @WebMethod(operationName="splitString")
  @WSDLDocumentation("Split a string into tokens")
  public abstract StringSplitType splitString( 
		  @WebParam(name="string") @XmlElement(required=true) String paramString,
		  @WebParam(name="delimiter") @XmlElement(required=true) String delimiter)
  	throws XServicesFault;
  
}